@@ -1,5 +1,5 @@
 #
-# mnemosyne2_cards.py <Peter.Bienstman@UGent.be>
+# mnemosyne2_cards.py <Peter.Bienstman@gmail.com>
 #
 
 import os
@@ -7,13 +7,13 @@ import sys
 import time
 import codecs
 import zipfile
-import cPickle
+import pickle
 
 from openSM2sync.log_entry import LogEntry
 from openSM2sync.log_entry import EventTypes
 from openSM2sync.text_formats.xml_format import XMLFormat
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.libmnemosyne.utils import MnemosyneError
 from mnemosyne.libmnemosyne.file_format import FileFormat
 
@@ -34,14 +34,14 @@ class Mnemosyne2Cards(FileFormat):
         if used_for_merging_dbs is True:
             metadata = {}
         else:
-            metadata = self.controller().show_export_metadata_dialog()
+            metadata = self.main_widget().show_export_metadata_dialog()
         if metadata is None:  # Cancelled.
             os.chdir(self.orig_dir)
             return -1
-        metadata_file = file("METADATA", "w")
-        for key, value in metadata.iteritems():
-            print >> metadata_file, key + ":" + \
-                value.strip().replace("\n", "<br>").encode("utf-8")
+        metadata_file = open("METADATA", "w", encoding="utf-8")
+        for key, value in metadata.items():
+            print(key + ":" + value.strip().replace("\n", "<br>"),
+                  file=metadata_file)
         metadata_file.close()
         db = self.database()
         w = self.main_widget()
@@ -57,7 +57,7 @@ class Mnemosyne2Cards(FileFormat):
             len(active_objects["media_filenames"]) + \
             len(active_objects["_card_ids"]) + \
             len(active_objects["_fact_ids"])
-        xml_file = file("cards.xml", "w")
+        xml_file = open("cards.xml", "w", encoding="utf-8")
         xml_format = XMLFormat()
         xml_file.write(xml_format.log_entries_header(number_of_entries))
         w.set_progress_range(number_of_entries)
@@ -67,8 +67,7 @@ class Mnemosyne2Cards(FileFormat):
             log_entry["type"] = EventTypes.ADDED_TAG
             log_entry["o_id"] = tag.id
             log_entry["name"] = tag.name
-            xml_file.write(xml_format.\
-                repr_log_entry(log_entry).encode("utf-8"))
+            xml_file.write(xml_format.repr_log_entry(log_entry))
             w.increase_progress(1)
         for fact_view_id in active_objects["fact_view_ids"]:
             fact_view = db.fact_view(fact_view_id, is_id_internal=False)
@@ -86,8 +85,7 @@ class Mnemosyne2Cards(FileFormat):
             log_entry["type_answer"] = repr(fact_view.type_answer)
             if fact_view.extra_data:
                 log_entry["extra"] = repr(fact_view.extra_data)
-            xml_file.write(xml_format.\
-                repr_log_entry(log_entry).encode("utf-8"))
+            xml_file.write(xml_format.repr_log_entry(log_entry))
             w.increase_progress(1)
         for card_type_id in active_objects["card_type_ids"]:
             card_type = db.card_type(card_type_id, is_id_internal=False)
@@ -107,25 +105,22 @@ class Mnemosyne2Cards(FileFormat):
                 repr(card_type.keyboard_shortcuts)
             if card_type.extra_data:
                 log_entry["extra"] = repr(card_type.extra_data)
-            xml_file.write(xml_format.\
-                repr_log_entry(log_entry).encode("utf-8"))
+            xml_file.write(xml_format.repr_log_entry(log_entry))
             w.increase_progress(1)
         for media_filename in active_objects["media_filenames"]:
             log_entry = LogEntry()
             log_entry["type"] = EventTypes.ADDED_MEDIA_FILE
             log_entry["fname"] = media_filename
-            xml_file.write(xml_format.\
-                repr_log_entry(log_entry).encode("utf-8"))
+            xml_file.write(str(xml_format.repr_log_entry(log_entry)))
             w.increase_progress(1)
         for _fact_id in active_objects["_fact_ids"]:
             fact = db.fact(_fact_id, is_id_internal=True)
             log_entry = LogEntry()
             log_entry["type"] = EventTypes.ADDED_FACT
             log_entry["o_id"] = fact.id
-            for fact_key, value in fact.data.iteritems():
+            for fact_key, value in fact.data.items():
                 log_entry[fact_key] = value
-            xml_file.write(xml_format.\
-                repr_log_entry(log_entry).encode("utf-8"))
+            xml_file.write(xml_format.repr_log_entry(log_entry))
             w.increase_progress(1)
         for _card_id in active_objects["_card_ids"]:
             card = db.card(_card_id, is_id_internal=True)
@@ -138,7 +133,7 @@ class Mnemosyne2Cards(FileFormat):
             log_entry["tags"] = ",".join([tag.id for tag in card.tags])
             if used_for_merging_dbs:
                 log_entry["c_time"] = card.creation_time
-                log_entry["m_time"] = card.modification_time 
+                log_entry["m_time"] = card.modification_time
                 log_entry["gr"] = card.grade
                 log_entry["e"] = card.easiness
                 log_entry["ac_rp"] = card.acq_reps
@@ -147,7 +142,7 @@ class Mnemosyne2Cards(FileFormat):
                 log_entry["ac_rp_l"] = card.acq_reps_since_lapse
                 log_entry["rt_rp_l"] = card.ret_reps_since_lapse
                 log_entry["l_rp"] = card.last_rep
-                log_entry["n_rp"] = card.next_rep               
+                log_entry["n_rp"] = card.next_rep
             else:
                 log_entry["gr"] = -1
                 log_entry["e"] = 2.5
@@ -160,8 +155,7 @@ class Mnemosyne2Cards(FileFormat):
                 log_entry["n_rp"] = -1
             if card.extra_data:
                 log_entry["extra"] = repr(card.extra_data)
-            xml_file.write(xml_format.\
-                repr_log_entry(log_entry).encode("utf-8"))
+            xml_file.write(xml_format.repr_log_entry(log_entry))
             w.increase_progress(1)
         xml_file.write(xml_format.log_entries_footer())
         xml_file.close()
@@ -197,7 +191,7 @@ class Mnemosyne2Cards(FileFormat):
         os.chdir(self.orig_dir)
         w.close_progress()
 
-    def do_import(self, filename, extra_tag_names=None, show_metadata=True):
+    def do_import(self, filename, extra_tag_names="", show_metadata=True):
         FileFormat.do_import(self, filename, extra_tag_names)
         if not extra_tag_names:
             extra_tags = []
@@ -205,26 +199,28 @@ class Mnemosyne2Cards(FileFormat):
             extra_tags = [self.database().get_or_create_tag_with_name(\
                 tag_name.strip()) for tag_name in extra_tag_names.split(",")]
         self.database().set_extra_tags_on_import(extra_tags)
-        # Extract zipfile. 
+        # Extract zipfile.
         w = self.main_widget()
         w.set_progress_text(_("Decompressing..."))
         zip_file = zipfile.ZipFile(filename, "r")
         zip_file.extractall(self.database().media_dir())
-        # Show metadata.          
+        # Show metadata.
         metadata_filename = os.path.join(\
                 self.database().media_dir(), "METADATA")
         if show_metadata:
             metadata = {}
-            for line in file(metadata_filename):
+            for line in open(metadata_filename, encoding="utf-8"):
                 key, value = line.split(":", 1)
                 metadata[key] = value.replace("<br>", "\n")
-            self.controller().show_export_metadata_dialog(metadata, read_only=True)
+            self.main_widget().show_export_metadata_dialog(\
+                metadata, read_only=True)
         # Parse XML.
         w.set_progress_text(_("Importing cards..."))
         self.database().card_types_to_instantiate_later = set()
         xml_filename = os.path.join(self.database().media_dir(), "cards.xml")
-        element_loop = XMLFormat().parse_log_entries(file(xml_filename, "r"))
-        number_of_entries = int(element_loop.next())
+        element_loop = XMLFormat().parse_log_entries(\
+            open(xml_filename, "r", encoding="utf-8"))
+        number_of_entries = int(next(element_loop))
         if number_of_entries == 0:
             return
         w.set_progress_range(number_of_entries)
@@ -234,7 +230,7 @@ class Mnemosyne2Cards(FileFormat):
             w.increase_progress(1)
         w.set_progress_value(number_of_entries)
         if len(self.database().card_types_to_instantiate_later) != 0:
-            raise RuntimeError, _("Missing plugins for card types.")
+            raise RuntimeError(_("Missing plugins for card types."))
         os.remove(xml_filename)
         os.remove(metadata_filename)
         w.close_progress()
