@@ -1,5 +1,5 @@
 #
-# spanish.py <Peter.Bienstman@UGent.be>
+# spanish.py <Peter.Bienstman@gmail.com>
 #
 
 from mnemosyne.libmnemosyne.gui_translator import _
@@ -10,6 +10,6 @@ class Spanish(Language):
 
     name = _("Spanish")
     used_for = "es"
-    sublanguages = {"es-ES", _("Spanish (Spain)"),
-                     "es-US", _("Spanish (US)")}
+    sublanguages = {"es-ES": _("Spanish (Spain)"),
+                     "es-US": _("Spanish (US)")}
     feature_description = _("Google translation and text-to-speech.")
