@@ -1,20 +1,19 @@
 #
-# export_dlg.py <Peter.Bienstman@UGent.be>
+# export_dlg.py <Peter.Bienstman@gmail.com>
 #
 
 import os
-from PyQt4 import QtGui, QtCore
+from PyQt6 import QtGui, QtCore, QtWidgets
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.pyqt_ui.ui_export_dlg import Ui_ExportDlg
 from mnemosyne.libmnemosyne.ui_components.dialogs import ExportDialog
 
 
-class ExportDlg(QtGui.QDialog, Ui_ExportDlg, ExportDialog):
+class ExportDlg(QtWidgets.QDialog, ExportDialog, Ui_ExportDlg):
 
-    def __init__(self, component_manager):
-        ExportDialog.__init__(self, component_manager)
-        QtGui.QDialog.__init__(self, self.main_widget())
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         # File formats.
         i = 0
@@ -30,19 +29,18 @@ class ExportDlg(QtGui.QDialog, Ui_ExportDlg, ExportDialog):
             self.file_formats.setCurrentIndex(current_index)
 
     def file_format_changed(self):
-        filename = unicode(self.filename_box.text())
+        filename = self.filename_box.text()
         if "." in filename:
-            filename = old_filename.rsplit(".")[0] + self.format().extension
+            filename = filename.rsplit(".")[0] + self.format().extension
             self.filename_box.setText(filename)
 
     def activate(self):
         ExportDialog.activate(self)
-        self.exec_()
+        self.exec()
 
     def format(self):
         for _format in self.component_manager.all("file_format"):
-            if _(_format.description) == \
-                unicode(self.file_formats.currentText()):
+            if _(_format.description) == self.file_formats.currentText():
                 return _format
 
     def browse(self):
@@ -54,13 +52,13 @@ class ExportDlg(QtGui.QDialog, Ui_ExportDlg, ExportDialog):
             self.config()["export_dir"] = os.path.dirname(filename)
 
     def accept(self):
-        filename = unicode(self.filename_box.text())
+        filename = self.filename_box.text()
         if not filename:
-            return QtGui.QDialog.accept(self)
+            return QtWidgets.QDialog.accept(self)
         if not filename.endswith(self.format().extension):
             filename += self.format().extension
         self.config()["export_format"] = str(type(self.format()))
         result = self.format().do_export(filename)
         if result != -1:  # Cancelled.
             self.main_widget().show_information(_("Done!"))
-        QtGui.QDialog.accept(self)
+        QtWidgets.QDialog.accept(self)
