@@ -3,18 +3,48 @@
 #
 
 import os
-from PyQt4 import QtGui, QtCore
+from PyQt6 import QtGui, QtCore, QtWidgets
 
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
+from mnemosyne.pyqt_ui.qt_worker_thread import \
+    QtWorkerThread, QtGuiThread
 from mnemosyne.pyqt_ui.ui_import_dlg import Ui_ImportDlg
 from mnemosyne.libmnemosyne.ui_components.dialogs import ImportDialog
 
 
-class ImportDlg(QtGui.QDialog, Ui_ImportDlg, ImportDialog):
+answer = None
+mutex = QtCore.QMutex()
+dialog_closed = QtCore.QWaitCondition()
 
-    def __init__(self, component_manager):
-        ImportDialog.__init__(self, component_manager)
-        QtGui.QDialog.__init__(self, self.main_widget())
+
+class ImportThread(QtWorkerThread):
+
+    show_export_metadata_signal = QtCore.pyqtSignal(dict, bool)
+
+    def show_export_metadata_dialog(self, metadata, read_only=True):
+        global answer
+        mutex.lock()
+        answer = None
+        self.show_export_metadata_signal.emit(metadata, read_only)
+        if not answer:
+            dialog_closed.wait(mutex)
+        mutex.unlock()
+
+    def __init__(self, filename, format, extra_tag_names, **kwds):
+        super().__init__(**kwds)
+        self.filename = filename
+        self.format = format
+        self.extra_tag_names = extra_tag_names
+
+    def do_work(self):
+        self.mnemosyne.config()["import_format"] = str(type(self.format))
+        self.format.do_import(self.filename, self.extra_tag_names)
+
+
+class ImportDlg(QtWidgets.QDialog, QtGuiThread, ImportDialog, Ui_ImportDlg):
+
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         # File formats.
         i = 0
@@ -51,12 +81,11 @@ class ImportDlg(QtGui.QDialog, Ui_ImportDlg, ImportDialog):
 
     def activate(self):
         ImportDialog.activate(self)
-        self.exec_()
+        self.exec()
 
     def format(self):
         for _format in self.component_manager.all("file_format"):
-            if _(_format.description) == \
-                unicode(self.file_formats.currentText()):
+            if _(_format.description) == self.file_formats.currentText():
                 return _format
 
     def browse(self):
@@ -68,14 +97,28 @@ class ImportDlg(QtGui.QDialog, Ui_ImportDlg, ImportDialog):
             self.config()["import_dir"] = os.path.dirname(filename)
 
     def accept(self):
-        filename = unicode(self.filename_box.text())
+        filename = self.filename_box.text()
         if filename and os.path.exists(filename):
-            extra_tag_names = unicode(self.tags.currentText())
+            extra_tag_names = self.tags.currentText()
             self.config()["import_extra_tag_names"] = extra_tag_names
             if not extra_tag_names:
-                extra_tag_names = None
-            self.config()["import_format"] = str(type(self.format()))
-            self.format().do_import(filename, extra_tag_names)
-            QtGui.QDialog.accept(self)
+                extra_tag_names = ""
+            self.worker_thread = ImportThread(filename,
+                self.format(), extra_tag_names, mnemosyne=self)
+            self.worker_thread.show_export_metadata_signal.connect(\
+                self.threaded_show_export_metadata)
+            self.run_worker_thread()
         else:
             self.main_widget().show_error(_("File does not exist."))
+
+    def threaded_show_export_metadata(self, metadata, read_only):
+        global answer
+        mutex.lock()
+        self.true_main_widget.show_export_metadata_dialog(metadata, read_only)
+        answer = True
+        dialog_closed.wakeAll()
+        mutex.unlock()
+
+    def work_ended(self):
+        self.main_widget().close_progress()
+        QtWidgets.QDialog.accept(self)
