@@ -1,45 +1,22 @@
 #
 # client.py - Max Usachev <maxusachev@gmail.com>
 #             Ed Bartosh <bartosh@gmail.com>
-#             Peter Bienstman <Peter.Bienstman@UGent.be>
+#             Peter Bienstman <Peter.Bienstman@gmail.com>
 #
 
 import os
 import socket
-import urllib
+import urllib.request, urllib.parse, urllib.error
 import tarfile
-import httplib
+import http.client
 
-from partner import Partner
-from text_formats.xml_format import XMLFormat
-from utils import SyncError, SeriousSyncError
-from utils import path_exists, path_getsize, path_join, traceback_string
-
-# Avoid delays caused by Nagle's algorithm.
-# http://www.cmlenz.net/archives/2008/03/python-httplib-performance-problems
-
-realsocket = socket.socket
-def socketwrap(family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0):
-    sockobj = realsocket(family, type, proto)
-    sockobj.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-    return sockobj
-socket.socket = socketwrap
-
-# Buffer the response socket.
-# http://mail.python.org/pipermail/python-bugs-list/2006-September/035156.html
-# Fix included here for systems running Python 2.5.
-
-class HTTPResponse(httplib.HTTPResponse):
-
-    def __init__(self, sock, **kw):
-        httplib.HTTPResponse.__init__(self, sock, **kw)
-        self.fp = sock.makefile("rb") # Was unbuffered: sock.makefile("rb", 0)
-
-httplib.HTTPConnection.response_class = HTTPResponse
+from .partner import Partner
+from .text_formats.xml_format import XMLFormat
+from .utils import SyncError, SeriousSyncError, traceback_string
 
 # Register binary formats.
 
-from binary_formats.mnemosyne_format import MnemosyneFormat
+from .binary_formats.mnemosyne_format import MnemosyneFormat
 BinaryFormats = [MnemosyneFormat]
 
 
@@ -96,8 +73,8 @@ class Client(Partner):
 
         # If we haven't done so, determine whether we're behind a proxy.
         if self.behind_proxy is None:
-            import urllib
-            proxies = urllib.getproxies()
+            import urllib.request, urllib.parse, urllib.error
+            proxies = urllib.request.getproxies()
             if "http" in proxies:
                 self.behind_proxy = True
                 self.proxy = proxies["http"]
@@ -105,18 +82,18 @@ class Client(Partner):
                 self.behind_proxy = False
         # Create a new connection or reuse an existing one.
         if self.behind_proxy:
-            httplib.HTTPConnection._http_vsn = 10
-            httplib.HTTPConnection._http_vsn_str = "HTTP/1.0"
+            http.client.HTTPConnection._http_vsn = 10
+            http.client.HTTPConnection._http_vsn_str = "HTTP/1.0"
             if self.proxy is not None:
-                self.con = httplib.HTTPConnection(self.proxy, self.port)
+                self.con = http.client.HTTPConnection(self.proxy, self.port)
             else:  # Testsuite has set self.behind_proxy to True to simulate
                 # being behind a proxy.
-                self.con = httplib.HTTPConnection(self.server, self.port)
+                self.con = http.client.HTTPConnection(self.server, self.port)
         else:
-            httplib.HTTPConnection._http_vsn = 11
-            httplib.HTTPConnection._http_vsn_str = "HTTP/1.1"
+            http.client.HTTPConnection._http_vsn = 11
+            http.client.HTTPConnection._http_vsn_str = "HTTP/1.1"
             if not self.con:
-                self.con = httplib.HTTPConnection(self.server, self.port)
+                self.con = http.client.HTTPConnection(self.server, self.port)
 
     def url(self, url_string):
         if self.behind_proxy and self.proxy:
@@ -127,6 +104,7 @@ class Client(Partner):
         try:
             self.server = socket.gethostbyname(server)
             self.port = port
+            backup_file = None
             if self.do_backup:
                 self.ui.set_progress_text("Creating backup...")
                 backup_file = self.database.backup()
@@ -139,7 +117,7 @@ class Client(Partner):
             # Always create media files, otherwise they are not synced across.
             self.ui.set_progress_text("Dynamically creating media files...")
             self.database.dynamically_create_media_files()
-            # Set timeout long enough for e.g. a slow NAS waking from 
+            # Set timeout long enough for e.g. a slow NAS waking from
             # hibernation.
             socket.setdefaulttimeout(60)
             self.login(username, password)
@@ -186,24 +164,25 @@ class Client(Partner):
                 # Upload local changes and check for conflicts.
                 result = self.put_client_log_entries()
                 if result == "OK":
+                    # We always need to put the client media files, regardless
+                    # of self.check_for_edited_local_media_files, as there could
+                    # be entirely new media files.
                     self.put_client_media_files()
                     self.get_server_media_files()
                     self.get_server_log_entries()
                     self.get_sync_finish()
                 else:
                     self.resolve_conflicts()
+            self.ui.close_progress()
             self.ui.show_information("Sync finished!")
-        except Exception, exception:
+        except Exception as exception:
             self.ui.close_progress()
             serious = True
-            if type(exception) == type(socket.gaierror()):
+            if type(exception) == type(socket.timeout()):
+                self.ui.show_error("Timeout while waiting for server!")
+            elif type(exception) == type(socket.gaierror()):
                 self.ui.show_error("Could not find server!")
                 serious = False
-            elif type(exception) == type(socket.error()):
-                self.ui.show_error("Could not connect to server!")
-                serious = False
-            elif type(exception) == type(socket.timeout()):
-                self.ui.show_error("Timeout while waiting for server!")
             elif type(exception) == type(SyncError()):
                 self.ui.show_error(str(exception))
                 serious = False
@@ -214,35 +193,35 @@ class Client(Partner):
             if serious and self.do_backup:
                 # Only serious errors should result in the need for a full
                 # sync next time.
-                self.ui.show_error("Sync failed, restoring from backup. " + \
-                    "The next sync will need to be a full sync.")
+                self.ui.show_error(\
+                    "Sync failed, the next sync will be a full sync.")
                 if backup_file:
                     self.database.restore(backup_file)
         finally:
-            
+
             # TMP: preparation for future fix
             #self.get_sync_cancel()
-            
-            
+
+
             if self.con:
                 self.con.close()
             self.ui.close_progress()
 
-    def supports_binary_upload(self):        
+    def supports_binary_upload(self):
         return self.capabilities == "mnemosyne_dynamic_cards" and \
-            self.interested_in_old_reps and self.store_pregenerated_data \
-            and self.program_name == self.server_info["program_name"] and \
-            self.program_version == self.server_info["program_version"]
+            self.interested_in_old_reps and self.store_pregenerated_data and \
+            self.program_name == self.server_info["program_name"] and \
+            self.database.version == self.server_info["database_version"]
 
     def resolve_conflicts(self, restored_from_backup=False):
         if restored_from_backup:
             message = "The database was restored from a backup, either " + \
-                "automatically because of an aborted sync or manually by " + \
-                "the user.\nFor safety, a full sync needs to happen and " + \
-                "you need to choose which copy of the database to keep and " + \
-                "which copy to discard.\n"
+    "automatically because of an aborted sync or manually by " + \
+    "the user.\nFor safety, a full sync of cards and history needs to happen and " + \
+    "you need to choose which copy of the database to keep and " + \
+    "which copy to discard.\n"
         else:
-            message = "Conflicts detected during sync! This typically happens if you review the same card on both machines. Choose which version to keep and which version to discard."
+            message = "Conflicts detected during sync! This typically happens if you review the same card on both machines. Choose which version of the database (cards + history) to keep and which version to discard."
         # Ask for conflict resolution direction.
         if self.supports_binary_upload():
             result = self.ui.show_question(message,
@@ -250,9 +229,7 @@ class Client(Partner):
             results = {0: "KEEP_LOCAL", 1: "KEEP_REMOTE", 2: "CANCEL"}
             result = results[result]
         else:
-            message += " " + "Your client only stores part of the server " + \
-                "database or uses a different software version, " + \
-                "so you can only fetch the remote version."
+            message += " " + "Your local client uses a different software version or only stores part of the remote server database, so you can only fetch the remote version."
             result = self.ui.show_question(message,
                 "Fetch remote version", "Cancel", "")
             results = {0: "KEEP_REMOTE", 1: "CANCEL"}
@@ -280,8 +257,9 @@ class Client(Partner):
 
     def _check_response_for_errors(self, response, can_consume_response=True):
         # Check for non-Mnemosyne error messages.
-        if response.status != httplib.OK:
-            raise SeriousSyncError("Internal server error:\n" + response.read())
+        if response.status != http.client.OK:
+            raise SeriousSyncError("Internal server error:\n" + \
+                                   str(response.read(), "utf-8"))
         if can_consume_response == False:
             return
         # Check for Mnemosyne error messages.
@@ -318,10 +296,18 @@ class Client(Partner):
             self.request_connection()
             self.con.request("PUT", self.url("/login"),
                 self.text_format.repr_partner_info(client_info).\
-                encode("utf-8") + "\n")
+                encode("utf-8") + b"\n")
             response = self.con.getresponse()
-        except Exception, e:
-            raise SyncError("Could not connect to server!")
+        except socket.gaierror:
+            raise SyncError("Could not find server!")
+        except socket.timeout:
+            raise SyncError("Timeout when trying to connect to server!")
+        except socket.error as e:
+            raise SyncError("Socket error: " + str(e))
+        except Exception as e:
+            print(e)
+            raise SyncError("Could not connect to server!\n\n" \
+                            + traceback_string())
         # Check for errors, but don't force a restore from backup if we can't
         # login.
         try:
@@ -329,7 +315,7 @@ class Client(Partner):
                 response, can_consume_response=False)
         except SeriousSyncError:
             raise SyncError("Logging in: server error.")
-        response = response.read()
+        response = str(response.read(), "utf-8")
         if "message" in response:
             message, traceback = self.text_format.parse_message(response)
             message = message.lower()
@@ -339,7 +325,7 @@ class Client(Partner):
                 raise SyncError("Wrong username or password.")
             if "cycle" in message:
                 raise SyncError(\
-                    "Sync cycle detected. Sync through intermediate partner.")
+"Sync cycle detected. Please always sync with the same server. Backup and delete the database and resync from scratch if necessary.")
             if "same machine ids" in message:
                 raise SyncError(\
 "You have manually copied the data directory before sync. Sync needs to start from an empty database.")
@@ -354,10 +340,7 @@ class Client(Partner):
                 "Backup then delete the local database and try again.")
         if self.server_info["database_version"] != client_info["database_version"]:
             raise SyncError("Error: database version mismatch.\n" + \
-                "Upgrade client and server to latest version.")
-        if self.server_info["program_version"] != client_info["program_version"]:
-            raise SyncError("Error: Mnemosyne version mismatch.\n" + \
-                "Upgrade client and server to latest version.")           
+                "Make sure you are running the latest Mnemosyne version on both devices involved in the sync.")
         self.database.create_if_needed_partnership_with(\
             self.server_info["machine_id"])
         self.database.merge_partners(self.server_info["partners"])
@@ -445,7 +428,7 @@ class Client(Partner):
 
     def _download_log_entries(self, stream):
         element_loop = self.text_format.parse_log_entries(stream)
-        number_of_entries = int(element_loop.next())
+        number_of_entries = int(next(element_loop))
         if number_of_entries == 0:
             return
         self.ui.set_progress_range(number_of_entries)
@@ -520,7 +503,7 @@ class Client(Partner):
     def put_client_media_files(self, reupload_all=False):
         self.ui.set_progress_text("Sending media files...")
         # Get list of filenames in the format <mediadir>/<filename>, i.e.
-        # relative to the data_dir. Note we always use / internally. 
+        # relative to the data_dir. Note we always use / internally.
         subdir = os.path.basename(self.database.media_dir())
         if reupload_all:
             filenames = [subdir + "/" + filename for filename in \
@@ -532,18 +515,18 @@ class Client(Partner):
         # Calculate file size and upload.
         total_size = 0
         for filename in filenames:
-            total_size += path_getsize(os.path.join(\
+            total_size += os.path.getsize(os.path.join(\
                 self.database.data_dir(), filename))
         self.put_client_binary_files(filenames, total_size)
         self.ui.close_progress()
-          
-    def put_client_archive_files(self):  
+
+    def put_client_archive_files(self):
         archive_dir = os.path.join(self.database.data_dir(), "archive")
         if not os.path.exists(archive_dir):
             return
         # Get list of filenames in the format "archive"/<filename>, i.e.
-        # relative to the data_dir. Note we always use / internally. 
-        self.ui.set_progress_text("Sending archive files...")       
+        # relative to the data_dir. Note we always use / internally.
+        self.ui.set_progress_text("Sending archive files...")
         filenames = ["archive/" + filename for filename in \
                      os.listdir(archive_dir) if os.path.isfile\
                      (os.path.join(archive_dir, filename))]
@@ -553,20 +536,21 @@ class Client(Partner):
             total_size += os.path.getsize(os.path.join(\
                 self.database.data_dir(), filename))
         self.put_client_binary_files(filenames, total_size)
-        self.ui.close_progress()        
-            
-    def put_client_binary_files(self, filenames, total_size):   
+        self.ui.close_progress()
+
+    def put_client_binary_files(self, filenames, total_size):
         # Filenames are relative to the data_dir.
         self.ui.set_progress_range(total_size)
         self.ui.set_progress_update_interval(total_size/50)
         for filename in filenames:
             self.request_connection()
+            #print(filename.encode("utf-8", "surrogateescape"))
             self.con.putrequest("PUT",
                 self.url("/client_binary_file?session_token=%s&filename=%s" \
                 % (self.server_info["session_token"],
-                urllib.quote(filename.encode("utf-8"), ""))))
-            full_path = path_join(self.database.data_dir(), filename)
-            file_size = path_getsize(full_path)
+                urllib.parse.quote(filename.encode("utf-8"), ""))))
+            full_path = os.path.join(self.database.data_dir(), filename)
+            file_size = os.path.getsize(full_path)
             self.con.putheader("content-length", file_size)
             self.con.endheaders()
             for buffer in self.stream_binary_file(full_path, progress_bar=False):
@@ -594,12 +578,12 @@ class Client(Partner):
             response.read()
             return
         filenames = []
-        for filename in response.read().split("\n"):
-            filenames.append(unicode(filename, "utf-8"))
+        for filename in response.read().split(b"\n"):
+            filenames.append(str(filename, "utf-8"))
         self.ui.set_progress_text("Getting media files...")
-        self.get_server_binary_files(filenames, total_size)    
+        self.get_server_binary_files(filenames, total_size)
         self.ui.close_progress()
-  
+
     def get_server_archive_files(self):
         self.ui.set_progress_text("Getting list of archive files to download...")
         # Get list of names of all archive files to download.
@@ -617,21 +601,21 @@ class Client(Partner):
             response.read()
             return
         filenames = []
-        for filename in response.read().split("\n"):
-            filenames.append(unicode(filename, "utf-8"))
+        for filename in response.read().split(b"\n"):
+            filenames.append(str(filename, "utf-8"))
         self.ui.set_progress_text("Getting archive files...")
-        self.get_server_binary_files(filenames, total_size)    
-        self.ui.close_progress()  
-               
+        self.get_server_binary_files(filenames, total_size)
+        self.ui.close_progress()
+
     def get_server_binary_files(self, filenames, total_size):
         self.ui.set_progress_range(total_size)
-        self.ui.set_progress_update_interval(total_size/50)  
+        self.ui.set_progress_update_interval(total_size/50)
         for filename in filenames:
             self.request_connection()
             self.con.request("GET",
                 self.url("/server_binary_file?session_token=%s&filename=%s" \
                 % (self.server_info["session_token"],
-                urllib.quote(filename.encode("utf-8"), ""))))
+                urllib.parse.quote(filename.encode("utf-8"), ""))))
             response = self.con.getresponse()
             self._check_response_for_errors(response,
                 can_consume_response=False)
@@ -651,7 +635,7 @@ class Client(Partner):
         self.request_connection()
         session_token = self.server_info.get("session_token", "none")
         self.con.request("GET", self.url("/sync_cancel?session_token=%s" \
-            % (session_token, )), headers={"connection": "close"})          
+            % (session_token, )), headers={"connection": "close"})
         self._check_response_for_errors(self.con.getresponse())
 
     def get_sync_finish(self):
