@@ -1,15 +1,15 @@
 #
-# qt_web_server.py <Peter.Bienstman@UGent.be>
+# qt_web_server.py <Peter.Bienstman@gmail.com>
 #
 
 import os
 import sys
 import socket
 
-from PyQt5 import QtCore
+from PyQt6 import QtCore
 
 from mnemosyne.libmnemosyne import Mnemosyne
-from mnemosyne.libmnemosyne.translator import _
+from mnemosyne.libmnemosyne.gui_translator import _
 from mnemosyne.web_server.web_server import WebServer
 from mnemosyne.libmnemosyne.component import Component
 from mnemosyne.libmnemosyne.utils import traceback_string
